# Generated by Django 6.0.3 on 2026-03-18 05:02

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="BudgetSnapshot",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("scope_key", models.CharField(db_index=True, max_length=255)),
                ("node_id", models.CharField(max_length=255)),
                ("stats_json", models.TextField()),
                ("updated_at", models.DateTimeField(auto_now=True)),
            ],
            options={
                "unique_together": {("scope_key", "node_id")},
            },
        ),
    ]
